@@ -22,6 +22,8 @@
 #ifndef add_standard_users_H
 #   define add_standard_users_H
 
-int add_standard_users();
+extern struct w *widgets;
+
+int add_standard_users(struct w *widgets);
 
 #endif
