@@ -31,6 +31,7 @@
 #include "show_info.h"
 #include "add_user.h"
 #include "functions.h"
+#include "chars_are_digits.h"
 #include "dir_treeview_funcs.h"
 #include "system_defines.h"
 #include "populate_users.h"
@@ -66,15 +67,78 @@ extern int global_dir_error;
 extern char global_version[1024];
 
 
+extern gchar *GP_PASSWD_BUF;
+extern gchar *GP_GROUP_BUF;
+
+
+
+/* Get the highest virtual user number +1 */
+long get_virtual_hnumber()
+{
+    FILE *fp;
+    char *line, *buf, *tmp;
+    gchar *info;
+    long file_size = 0;
+    long count = 0, tmpnumber = 0, hnumber = 0;
+
+    if((fp=fopen(GP_PASSWD_BUF, "r"))==NULL)
+    {
+	info = g_strdup_printf(_("Failed reading file: %s\n"), GP_PASSWD_BUF);
+	show_info(info);
+	g_free(info);
+
+    	return 1001;
+    }
+    fseek(fp, 0, SEEK_END);
+    file_size = ftell(fp);
+    rewind(fp);
+    
+    line = allocate(file_size+1);
+    buf  = allocate(file_size+1);
+
+    if( file_size > 1 )
+    while(fgets(line, file_size, fp)!=NULL)
+    {
+	if( strlen(line) < 3 )
+	  continue;
+	  
+	// Get value from 3:rd or 4:th section
+	tmp = strtok(line, ":");
+
+	count = 0; tmpnumber = 0;	
+	while( tmp!=NULL )
+	{
+	    count++;
+	
+	    if( chars_are_digits(tmp) && (count == 3 || count == 4) )
+	    {
+		tmpnumber = atoi(tmp);
+		if( tmpnumber > hnumber )
+		  hnumber = tmpnumber + 1;
+	    }
+	
+	    tmp = strtok(NULL, ":");
+	}
+    }
+    fclose(fp);
+
+    if( hnumber < 1000 )
+      hnumber = 1000;
+
+    return hnumber+1;
+}
+
+
 
 void add_user(struct w *widgets)
 {
     /* Adds a new user to the selected server */
     FILE *fp;
-    long conf_size = 0;
+    long hnumber = 0, conf_size = 0;
     long profile_size = 0;
     char *old_buffer, *new_buffer;
     char *user_check, *address_buffer, *port_buffer;
+    const char *encrypted_pass;
     int length=0, limit_access=0, user_added=0;
     int found_server=0, standard_server=0;
     gchar *utf8=NULL;
@@ -219,13 +283,13 @@ void add_user(struct w *widgets)
     while(fgets(old_buffer, conf_size, fp)!=NULL)
     {
 	/* The correct server address is found */
-        if( ! found_server && ! strcmp(old_buffer, address_buffer) ) 
+        if( ! found_server && strcmp(old_buffer, address_buffer) == 0 ) 
 	{
 	    /* Lets see if its the same port as the selected one */
 	    while(fgets(old_buffer, conf_size, fp)!=NULL)
 	    {
 		if( strstr(old_buffer, "Port") 
-		&& ! strcmp(old_buffer, port_buffer) )
+		&&  strcmp(old_buffer, port_buffer) == 0 )
 		{
 		    found_server = 1;
 		    break;
@@ -263,9 +327,9 @@ void add_user(struct w *widgets)
     if( conf_size > 1 )
     while(fgets(old_buffer, conf_size, fp)!=NULL)
     {
-	if( ! strcmp(old_buffer, user_check) 
-	&&  ! strstr(old_buffer, "AllowUser")
-	&&  ! strstr(old_buffer, "FakeUser") )
+	if( strcmp(old_buffer, user_check)  == 0
+	&&  strstr(old_buffer, "AllowUser") == 0
+	&&  strstr(old_buffer, "FakeUser")  == 0)
 	{
 	    info = g_strdup_printf(_("Failed adding user: %s\nThe user already exists in this server.\n"), username);
  	    show_info(info);
@@ -295,124 +359,216 @@ void add_user(struct w *widgets)
 
 
 
-    /* The selected shell is false, add the group and the user with a false shell and home /dev/null */
-    if( strstr(shell, "nologin") || strstr(shell, "false") || strstr(shell, "dev/null") )
+
+
+    /* Virtual users */
+    if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->server_set_check_virtual)) )
     {
-	/* All supported systems but darwin */
-#ifndef USE_DARWIN
-
-	/* Add the group if it doesnt exist */
-	if( ! group_exists(group) )
+	if( ! strstr(GP_PASSWD_BUF, "/users/") )
 	{
-	    cmd = g_strdup_printf("%s '%s'", ADDGROUP, group);
-	    if( ! run_command(cmd) )
-	    {
-		info = g_strdup_printf(_("Error adding group: %s\n"), group);
- 		show_info(info);
-		g_free(info);
-	    }
-	    g_free(cmd);
+	    printf("add_user.c: Safeguard activated\n");
+	    return;
 	}
 
-	/* Add the user to this group if it doesnt exist */
-        if( ! user_exists(username) )
-        {
-	    cmd = g_strdup_printf("%s '%s' -g '%s' -d /dev/null -c '%s' -s %s", ADDUSER, username, group, comment, shell);
-	    if( ! run_command(cmd) )
-	    {
-		info = g_strdup_printf(_("Failed adding user: %s\n"), username);
- 		show_info(info);
-		g_free(info);
-	    }
-	    else
-	      user_added = 1;
-
-	    g_free(cmd);
-	}
-
-#elif USE_DARWIN
-	/* Add the false user using darwins niutil commands (Darwin is out of sync) */
-	if( ! niutil_user_exists(username) )    
+	if( ! file_exists(GP_PASSWD_BUF) )
 	{
-	    if( ! niutil_useradd(username, shell) )
-	    {
-		info = g_strdup_printf(_("Failed adding user: %s\n"), username);
- 		show_info(info);
-		g_free(info);
-	    }
-	    else
-	      user_added = 1;
-	}
-#endif
-    }
-    else 	
-      {
-
-         /* The selected shell is not false add a real user account */
-#ifndef USE_DARWIN
-
-	/* Add the group if it doesnt exist */
-	if( ! group_exists(group) )
-	{
-	    cmd = g_strdup_printf("%s '%s'", ADDGROUP, group);
+	    cmd = g_strdup_printf("touch %s", GP_PASSWD_BUF);
 	    if( ! run_command(cmd) )
-	    {
-		info = g_strdup_printf(_("Failed adding group: %s\n"), group);
- 		show_info(info);
-		g_free(info);
-	    }
+	      printf("Error running command: %s\n", cmd);
 	    g_free(cmd);
 	}
 	
-	/* Add the user to this group if it doesnt exist */
-	if( ! user_exists(username) )
+	if( ! file_exists(GP_GROUP_BUF) )
 	{
-	    /* Add the user with a real shell to /USERSHOME/ UserName (was: -m -s) */
-	    cmd = g_strdup_printf("%s '%s' -g '%s' -d '%s%s' -c '%s' -s %s", ADDUSER, username, group, USERSHOME, username, comment, shell);
+	    cmd = g_strdup_printf("touch %s", GP_GROUP_BUF);
 	    if( ! run_command(cmd) )
-	    {
-		info = g_strdup_printf(_("Failed adding user: %s\n"), username);
- 		show_info(info);
-		g_free(info);
-	    }
-	    else
-	      user_added = 1;
-
+	      printf("Error running command: %s\n", cmd);
 	    g_free(cmd);
 	}
 
-#elif USE_DARWIN
-	 /* Add a real darwin user using the niutil commands (Darwin is out of sync) */
-	 if( ! niutil_user_exists(username) )    
-	 {
-	     if( ! niutil_useradd(username, shell) )
-	     {
-		info = g_strdup_printf(_("Failed adding user: %s\n"), username);
- 		show_info(info);
-		g_free(info);
-	     }
-	     else
-	       user_added = 1;
-	 }
-#endif
-    }
+	/* Chown /etc/gadmin-proftpd so that the server user/group has access */
+	cmd = g_strdup_printf("chown %s:%s %s", SERVER_USER, SERVER_GROUP, GP_APPCONFDIR);
+	if( ! run_command(cmd) )
+	  printf("Error running command: %s\n", cmd);
+	g_free(cmd);
 
+	/* Chown /etc/gadmin-proftpd and /etc/gadmin-proftpd/users directories */
+	cmd = g_strdup_printf("chown %s:%s %s/users", SERVER_USER, SERVER_GROUP, GP_APPCONFDIR);
+	if( ! run_command(cmd) )
+	  printf("Error running command: %s\n", cmd);
+	g_free(cmd);
+	
+        /* Append a user line to proftpd.passwd
+	   name:$1$5GQxXfZ3$4NcgFKj7M9UxbEGZEdAH8.:1002:1002::/home/name:shell */
+	encrypted_pass = encrypt_password(password);
+	if( encrypted_pass == NULL )
+	{
+	    printf("Crypt error\n");
+	    
+	    // FIXME: free some
 
-    /* Dont add anything if we couldnt add the system user */
-#ifndef USE_DARWIN
-    if( ! user_exists(username) )
-#elif USE_DARWIN
-    if( ! niutil_user_exists(username) )
-#endif
-    {
-	info = g_strdup_printf(_("The system user was not added because uppercase\nor language specific letters are not allowed.\n"));
- 	show_info(info);
+	    return;
+	}
+	
+	hnumber = get_virtual_hnumber();
+	
+	info = g_strdup_printf("%s:%s:%ld:%ld:%s:%s:%s\n", username, encrypted_pass, hnumber, hnumber, comment, "/dev/null", shell);	
+	
+	if((fp=fopen(GP_PASSWD_BUF, "a+"))==NULL)
+	{
+	    info = g_strdup_printf(_("Failed adding user: %s\n"), username);
+	    show_info(info);
+	    g_free(info);
+
+    	    return;
+	}
+	fputs(info, fp);
+	fclose(fp);
 	g_free(info);
+	
 
-	free(address_buffer);
-	free(port_buffer);
-	return;
+	/* Append to proftpd.group / group:x:1001 */
+	info = g_strdup_printf("%s:x:%ld\n", group, hnumber);	
+	
+	if((fp=fopen(GP_GROUP_BUF, "a+"))==NULL)
+	{
+	    info = g_strdup_printf(_("Failed adding group: %s\n"), username);
+	    show_info(info);
+	    g_free(info);
+
+    	    return;
+	}
+	fputs(info, fp);
+	fclose(fp);
+	g_free(info);
     }
+    else  /* End virtual users  */
+    {
+          /* Begin system users */
+
+
+
+	/* The selected shell is false, add the group and the user with a false shell and home /dev/null */
+	if( strstr(shell, "nologin") || strstr(shell, "false") || strstr(shell, "dev/null") )
+	{
+	    /* All supported systems but darwin */
+#ifndef USE_DARWIN
+
+	    /* Add the group if it doesnt exist */
+	    if( ! group_exists(group) )
+	    {
+		cmd = g_strdup_printf("%s '%s'", ADDGROUP, group);
+		if( ! run_command(cmd) )
+		{
+		    info = g_strdup_printf(_("Error adding group: %s\n"), group);
+ 		    show_info(info);
+		    g_free(info);
+		}
+		g_free(cmd);
+	    }
+
+	    /* Add the user to this group if it doesnt exist */
+    	    if( ! user_exists(username) )
+    	    {
+		cmd = g_strdup_printf("%s '%s' -g '%s' -d /dev/null -c '%s' -s %s", ADDUSER, username, group, comment, shell);
+		if( ! run_command(cmd) )
+		{
+		    info = g_strdup_printf(_("Failed adding user: %s\n"), username);
+ 		    show_info(info);
+		    g_free(info);
+		}
+		else
+	          user_added = 1;
+
+		g_free(cmd);
+	    }
+
+#elif USE_DARWIN
+	    /* Add the false user using darwins niutil commands (Darwin is out of sync) */
+	    if( ! niutil_user_exists(username) )    
+	    {
+		if( ! niutil_useradd(username, shell) )
+		{
+		    info = g_strdup_printf(_("Failed adding user: %s\n"), username);
+ 		    show_info(info);
+		    g_free(info);
+		}
+		else
+	          user_added = 1;
+	    }
+#endif
+	}
+	else 	
+        {
+
+            /* The selected shell is not false add a real user account */
+#ifndef USE_DARWIN
+
+	    /* Add the group if it doesnt exist */
+	    if( ! group_exists(group) )
+	    {
+		cmd = g_strdup_printf("%s '%s'", ADDGROUP, group);
+		if( ! run_command(cmd) )
+		{
+		    info = g_strdup_printf(_("Failed adding group: %s\n"), group);
+ 		    show_info(info);
+		    g_free(info);
+		}
+		g_free(cmd);
+	    }
+	
+	    /* Add the user to this group if it doesnt exist */
+	    if( ! user_exists(username) )
+	    {
+		/* Add the user with a real shell to /USERSHOME/ UserName (was: -m -s) */
+		cmd = g_strdup_printf("%s '%s' -g '%s' -d '%s%s' -c '%s' -s %s", ADDUSER, username, group, USERSHOME, username, comment, shell);
+		if( ! run_command(cmd) )
+		{
+		    info = g_strdup_printf(_("Failed adding user: %s\n"), username);
+ 		    show_info(info);
+		    g_free(info);
+		}
+		else
+	          user_added = 1;
+
+		g_free(cmd);
+	    }
+
+#elif USE_DARWIN
+	    /* Add a real darwin user using the niutil commands (Darwin is out of sync) */
+	    if( ! niutil_user_exists(username) )    
+	    {
+	        if( ! niutil_useradd(username, shell) )
+	        {
+		    info = g_strdup_printf(_("Failed adding user: %s\n"), username);
+ 		    show_info(info);
+		    g_free(info);
+	        }
+	        else
+	          user_added = 1;
+	    }
+#endif
+	}
+
+
+	/* Dont add anything if we couldnt add the system user */
+#ifndef USE_DARWIN
+	if( ! user_exists(username) )
+#elif USE_DARWIN
+	if( ! niutil_user_exists(username) )
+#endif
+	{
+	    info = g_strdup_printf(_("The system user was not added because uppercase\nor language specific letters are not allowed.\n"));
+ 	    show_info(info);
+	    g_free(info);
+
+	    free(address_buffer);
+	    free(port_buffer);
+	    return;
+	}
+
+    }   /* End of adding system or virtual users */
+
 
 
     /* Setup the users profile and create its directories */
@@ -582,7 +738,7 @@ void add_user(struct w *widgets)
 	strcat(new_buffer, old_buffer);
 
         if( ! standard_server && ! found_server 
-	&&  ! strcmp(old_buffer, address_buffer) )
+	&&  strcmp(old_buffer, address_buffer) == 0 )
 	{
 	    /* Lets see if this is the selected server */
 	    while(fgets(old_buffer, conf_size, fp)!=NULL)
@@ -592,7 +748,7 @@ void add_user(struct w *widgets)
 		/* This will expect the servers port on the second line ! 
 		 * else itll miss some vaules .. */
 		if( strstr(old_buffer, "Port") 
-		&& ! strcmp(old_buffer, port_buffer) )
+		&& strcmp(old_buffer, port_buffer) == 0 )
 		{
 		    found_server = 1;
 		    break;
@@ -645,22 +801,25 @@ void add_user(struct w *widgets)
     free(port_buffer);
     free(user_profile);
 
-
-    /* Password the user if it didnt exist before */
-    if( user_added )
+    /* Password the user if its not virtual */
+    if( ! gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->server_set_check_virtual)) )
     {
+	/* Password the user if it didnt exist before */
+	if( user_added )
+	{
 #ifndef USE_DARWIN
-         password_user(username, password);
+            password_user(username, password);
 #elif USE_DARWIN
-	 niutil_password_user(username, password);
+	    niutil_password_user(username, password);
 #endif
+	}
+	else
+        {
+	    info = g_strdup_printf(_("The system user \"%s\" already exists.\nThe user was added to this server but the password was not changed.\n"), username);
+ 	    show_info(info);
+	    g_free(info);
+        }
     }
-    else
-      {
-	  info = g_strdup_printf(_("The system user \"%s\" already exists.\nThe user was added to this server but the password was not changed.\n"), username);
- 	  show_info(info);
-	  g_free(info);
-      }
 
     /* Write the new configuration if the user profile was added. 
      * Since the user could have already existed we use limit_access */
