@@ -56,7 +56,7 @@ void apply_server_settings(struct w *widgets)
     int configuration_changed = 0;
     gint active_index;
     gchar *address_line, *port_line, *info;
-    gchar *settings, *settings_conf;
+    gchar *virtual_users, *settings, *settings_conf;
 
     char *change_srv_ident;
     char *old_buffer, *config, *temp_server, *conf_line, *address_buffer, *port_buffer;
@@ -121,11 +121,44 @@ void apply_server_settings(struct w *widgets)
     upload_bandwidth = gtk_entry_get_text(GTK_ENTRY(widgets->server_set_spinbutton[9]));
     download_bandwidth = gtk_entry_get_text(GTK_ENTRY(widgets->server_set_spinbutton[10]));
 
-
     /* Save the old server ident so we know if we should repopulate the server treeview */
     old_server_ident = g_strdup_printf("%s", server_ident);
 
 
+    /* First save the settings */
+    settings_conf = g_strdup_printf("%s/settings.conf", GP_APPCONFDIR);
+    if((fp=fopen(settings_conf, "w+"))==NULL)
+    {
+	g_free(settings_conf);
+	// Show popup
+        return;
+    }
+
+    if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->server_set_check_virtual)) )
+      virtual_users = g_strdup_printf("on");
+    else
+      virtual_users = g_strdup_printf("off");
+
+    settings = g_strconcat("\n",
+    "certificate_directory ", GP_APPCONFDIR, "/certs\n",
+    "random_username_length 6\n",
+    "random_password_length 6\n",
+    "randomize_case lower\n",
+    "useradd_homedir_path /var/ftp\n",
+    "html_path /var/www/html/ftp.htm\n",
+    "welcome_name welcome.msg\n",
+    "virtual_users ", virtual_users, "\n",
+    "\n",
+    NULL);
+    fputs(settings, fp);
+    fclose(fp);
+
+    g_free(settings);
+    g_free(settings_conf);
+    g_free(virtual_users);
+
+
+
     if( use_tls )
     {
 	/* Get values from the TLS entries */
@@ -577,9 +610,9 @@ void apply_server_settings(struct w *widgets)
 	free(config);
 
 	/* The server address, ident or port hasnt changed, dont populate server tab */
-	if( ! strcmp(global_server_address, server_name) 
-	&&  ! strcmp(old_server_ident, server_ident)
-	&&  ! strcmp(global_server_port, server_port)  )
+	if( strcmp(global_server_address, server_name) == 0
+	&&  strcmp(old_server_ident, server_ident)     == 0
+	&&  strcmp(global_server_port, server_port)    == 0 )
 	{}
 	else
         {
@@ -614,7 +647,7 @@ void apply_server_settings(struct w *widgets)
 	if( conf_size > 1 )
         while(fgets(old_buffer, conf_size, fp)!=NULL)
         {
-	    if( ! strcmp(old_buffer, address_line) )
+	    if( strcmp(old_buffer, address_line) == 0 )
 	    {
 		address_match = 1;
 
@@ -625,7 +658,7 @@ void apply_server_settings(struct w *widgets)
     		while(fgets(old_buffer, conf_size, fp)!=NULL)
 		    break;
 
-		if( address_match && ! strcmp(old_buffer, port_line) )
+		if( address_match && strcmp(old_buffer, port_line) == 0 )
 		{
 		    port_match = 1;
 		    
@@ -667,15 +700,9 @@ void apply_server_settings(struct w *widgets)
 
 	    fclose(fp);
 
-//	    free(old_buffer);
-//	    free(address_buffer);
-//	    free(temp_server);
-//	    free(port_buffer);
 	    free(change_srv_ident);
-//new	    free(conf_line);
-	    free(config);
-
             g_free(old_server_ident);
+	    free(config);
 
 	    printf("Broken...\n");
 	    return;
@@ -1040,9 +1067,9 @@ void apply_server_settings(struct w *widgets)
 	free(config);
 
 	/* The server address, ident or port hasnt changed, dont populate_servers */
-	if( ! strcmp(global_server_address, server_name) 
-	&&  ! strcmp(old_server_ident, server_ident)
-	&&  ! strcmp(global_server_port, server_port)  )
+	if( strcmp(global_server_address, server_name) == 0
+	&&  strcmp(old_server_ident, server_ident)     == 0
+	&&  strcmp(global_server_port, server_port)    == 0 )
 	{  }
 	else
 	{
@@ -1061,29 +1088,4 @@ void apply_server_settings(struct w *widgets)
     }   /* Vhost end */
 
 
-// FIXME: get widget numbers etc from old gadmin-proftpd
-
-    /* Save settings to settings.conf */
-    settings_conf = g_strdup_printf("%s/settings.conf", GP_APPCONFDIR);
-    if((fp=fopen(settings_conf, "w+"))==NULL)
-    {
-	g_free(settings_conf);
-	// Show popup
-        return;
-    }
-
-    settings = g_strconcat("\n",
-    "certificate_directory ", GP_APPCONFDIR, "/certs\n",
-    "random_username_length 6\n",
-    "random_password_length 6\n",
-    "randomize_case lower\n",
-    "useradd_homedir_path /var/ftp\n",
-    "html_path /var/www/html/ftp.htm\n",
-    "welcome_name welcome.msg\n",
-    "\n",
-    NULL);
-    fputs(settings, fp);
-    fclose(fp);
-    g_free(settings);
-    g_free(settings_conf);
 }
