@@ -35,6 +35,8 @@
 #include "import_functions.h"
 #include "create_import_window.h"
 
+extern gchar *GP_PASSWD_BUF;
+
 
 
 void create_import_window(struct w *widgets)
@@ -211,9 +213,9 @@ void create_import_window(struct w *widgets)
 
     /* Populate the import treeview with users */
 
-    if((fp=fopen(GP_PASSWD, "r"))==NULL)
+    if((fp=fopen(GP_PASSWD_BUF, "r"))==NULL)
     {
-	printf("Cant open passwd here:\n%s\n", GP_PASSWD);
+	printf("Cant open passwd here:\n%s\n", GP_PASSWD_BUF);
 	return;
     }
 
