@@ -20,24 +20,132 @@
 
 
 #include "../config.h"
+#include "gtk/gtk.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gtk/gtk.h>
 #include "gettext.h"
 #include "widgets.h"
+#include "allocate.h"
+#include "show_info.h"
 #include "make_settings_entries.h"
 #include "make_settings_combos.h"
 #include "make_settings_spinbuttons.h"
 #include "make_settings_progressbars.h"
+#include "make_settings_checkbuttons.h"
 #include "make_settings_labels.h"
 #include "make_settings_hseparators.h"
 #include "generate_cert.h"
 #include "create_server_settings.h"
+#include "populate_conf_tab.h"
 
 extern int use_tls;
 extern int use_ratio;
 
+gulong virtual_users_toggled_signal;
+
+
+/* Block or allow pam logins for system users 
+   1 to add restriction, 0 to remove the restriction. */
+void restrict_pam_conf(int add_del)
+{
+    FILE *fp;
+    long file_size = 0;
+    char *line, *new_conf;
+    gchar *pam_module_off;
+    
+    if((fp=fopen(PROFTPD_CONF, "r"))==NULL)
+    {
+	/* We dont want to add anything unless the conf exists. */
+        return;
+    }
+    fseek(fp, 0, SEEK_END);
+    file_size = ftell(fp);
+    rewind(fp);
+				    
+    line     = allocate(file_size+1);
+    new_conf = allocate(file_size+1024);
+
+    pam_module_off = g_strconcat("\n",
+    "<IfModule mod_auth_pam.c>\n",
+    "AuthPAM off\n",
+    "</IfModule>\n\n",
+    NULL);
+
+    if( file_size > 1 )
+    while(fgets(line, file_size, fp)!=NULL)
+    {
+	/* Remove the pam restriction */
+	if( line!=NULL && strstr(line, "<IfModule mod_auth_pam.c>") && add_del == 0 )
+        {
+	    while(fgets(line, file_size, fp)!=NULL)
+	    if( line!=NULL && strstr(line, "</IfModule>") )
+		break;
+	}
+	else /* Add pam restriction before the tls module */
+	if( line!=NULL && strstr(line, "<IfModule mod_tls.c>") && add_del == 1 )
+	{
+	    strcat(new_conf, pam_module_off);
+	    strcat(new_conf, line);
+	}
+	else
+	  strcat(new_conf, line);
+    }
+    fclose(fp);
+    free(line);
+    g_free(pam_module_off);
+
+    /* Write the new conf */
+    if((fp=fopen(PROFTPD_CONF, "w+"))==NULL)
+    {
+        return; 
+    }
+    fputs(new_conf, fp);
+    fclose(fp);
+    free(new_conf);
+}
+
+
+void virtual_users_toggled(GtkToggleButton *togglebutton, struct w *widgets)
+{
+    gchar *info;
+
+    if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton)) )
+    {
+	info = g_strconcat("\n",
+	_("Virtual user management activated.\n\n"),
+	_("Any system users you have created will no longer be able to log in.\n"),
+	
+	_("\nIf you also want to let system users login then remove entire\n"),
+	_("newly added AuthPAM off module restriction lines in the configuration tab.\n"),
+	
+	_("\nPress apply if this is what you want.\n\n"),
+	
+	_("Switch back if this is not what you want.\n"),
+	NULL);
+	show_info(info);
+	g_free(info);
+	
+	restrict_pam_conf(1);
+	populate_conf_tab(widgets);
+    }
+    else
+    {
+	info = g_strconcat("\n",
+	_("Virtual user management deactivated.\n\n"),
+
+	_("Any virtual users you have created will no longer be able to log in.\n"),
+	_("Press apply if this is what you want.\n\n"),
+	
+	_("Switch back if this is not what you want.\n"),
+	NULL);
+	show_info(info);
+	g_free(info);
+
+	restrict_pam_conf(0);
+	populate_conf_tab(widgets);
+    }
+}
 
 
 void create_server_settings(struct w *widgets)
@@ -212,27 +320,27 @@ void create_server_settings(struct w *widgets)
     utf8 = g_locale_to_utf8(combo_text, strlen(combo_text), NULL, NULL, NULL);
     gtk_combo_box_append_text(GTK_COMBO_BOX(widgets->server_set_combo[comb]), utf8);
     g_free(combo_text); g_free(utf8);
+
+    /* Leave these for now */
     a++; b++; comb++;
 
-    /* Default home directory */
-/*
-    widgets->server_set_entry[ent] = make_long_entry_with_label(GTK_TABLE(widgets->srv_set_table), _(" Default home directory: "), 0,1,a,b,200);
-    gtk_tooltips_set_tip(tooltips, widgets->server_set_entry[ent], _("This will be set as the users default home directory"), NULL);
-*/
     a++; b++; ent++;
 
-    /* Default upload directory */
-/*
-    widgets->server_set_entry[ent] = make_long_entry_with_label(GTK_TABLE(widgets->srv_set_table), _(" Default upload directory: "), 0,1,a,b,200);
-    gtk_tooltips_set_tip(tooltips, widgets->server_set_entry[ent], _("This will be set as the users default upload directory"), NULL);
-*/
     a++; b++; ent++;
 
+    /* Enable or disable virtual users */
+    widgets->server_set_check_virtual = make_checkbutton_with_label(GTK_TABLE(widgets->srv_set_table), _(" Enable virtual users: "), 0,1,a,b);
+    a++; b++;
+
+    /* Add a callback that switches to / from virtual user paths and repopulates */
+    virtual_users_toggled_signal = g_signal_connect(GTK_TOGGLE_BUTTON(widgets->server_set_check_virtual), "toggled",
+		                		 			    G_CALLBACK(virtual_users_toggled), widgets);
+
+
     /* Default statistics file */
     widgets->server_set_entry[ent] = make_long_entry_with_label(GTK_TABLE(widgets->srv_set_table), _(" Default HTML statistics filepath: "), 0,1,a,b,200);
     gtk_tooltips_set_tip(tooltips, widgets->server_set_entry[ent], _("This will be set as the default HTML statistics filepath, IE: /var/www/ftp.htm"), NULL);
     a++; b++; ent++;
-// End should be added to a settings window.
 
 
     ent++; // Leave this, deleted the welcome entry
@@ -433,11 +541,8 @@ void create_server_settings(struct w *widgets)
 	widgets->server_set_entry[ent] = make_long_entry_with_label(GTK_TABLE(widgets->srv_set_table), _(" Password: "), 0,1,a,b,200);
 	gtk_tooltips_set_tip(tooltips, widgets->server_set_entry[ent], _("A password"), NULL);
 	a++; b++; ent++;
-/*
-	// Challenge password. Not used anymore
-	widgets->server_set_entry[ent] = make_long_entry_with_label(GTK_TABLE(widgets->srv_set_table), _(" Challenge password: "), 0,1,a,b,200);
-	gtk_tooltips_set_tip(tooltips, widgets->server_set_entry[ent], _("A challenge password"), NULL);
-*/
+
+	/* Was challenge password, let it be for now */
 	a++; b++; ent++;
 
 	/* Number of encryption bits (Min value = 384) */
@@ -457,7 +562,7 @@ void create_server_settings(struct w *widgets)
 	a++; b++; ent++;
 
 	/* A progress bar */
-	widgets->gen_cert_progressbar = make_progressbar_with_label(GTK_TABLE(widgets->srv_set_table), _(" Generate a new selfsigned certificate: "), 0,1,a,b,210);
+	widgets->gen_cert_progressbar = make_progressbar_with_label(GTK_TABLE(widgets->srv_set_table), _(" Generate new certificate: "), 0,1,a,b,210);
 	gtk_widget_set_sensitive(widgets->gen_cert_progressbar, FALSE);
 
 	/* The progress bar looks nicer if its blue from start */
