@@ -110,7 +110,7 @@ void delete_server(struct w *widgets)
     {
 	/* Is this the correct server..name to delete */
         if( strstr("Virtualhost", (char *)global_server_type) && ! found_server
-	&& ! strcmp(old_buffer, address_buffer) && ! deleted_server )
+	&& strcmp(old_buffer, address_buffer) == 0 && ! deleted_server )
 	{
 	    strcpy(temp_vhost, old_buffer);
 	    while(fgets(old_buffer, conf_size, fp)!=NULL)
@@ -120,7 +120,7 @@ void delete_server(struct w *widgets)
 
 		strcat(temp_vhost, old_buffer);
 
-		if( strstr(old_buffer, "Port ") && ! strcmp(old_buffer, port_buffer) )
+		if( strstr(old_buffer, "Port ") && strcmp(old_buffer, port_buffer) == 0 )
 		{
 		    found_server = 1;
 		    deleted_server = 1;
