@@ -266,20 +266,26 @@ void delete_user(struct w *widgets)
 
     if( user_exists(username) )
     {
-	create_del_system_user_question(widgets);
-	gtk_widget_show_all(widgets->del_system_user_question_window);
+	/* Virtual users */
+	if( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widgets->server_set_check_virtual)) )
+	{
+	    delete_virtual_user(username);
+	}
+	else
+	{
+	    create_del_system_user_question(widgets);
+	    gtk_widget_show_all(widgets->del_system_user_question_window);
+	}
     }
-    else
-      {
-	  /* Update the user list and the user settings */
-	  populate_users(widgets);
 
-	  select_first_user(widgets);
+    /* Update the user list and the user settings */
+    populate_users(widgets);
 
-	  populate_user_settings(widgets);
+    select_first_user(widgets);
+
+    populate_user_settings(widgets);
     
-	  populate_conf_tab(widgets);
-      }
+    populate_conf_tab(widgets);
 
     /* Update the server */
     reread_conf(widgets);
