@@ -47,6 +47,16 @@
 #endif
 
 
+/* Settings value. Written on apply server settings. */
+extern int use_virtual_users; 
+
+extern gchar *GP_PASSWD_BUF;
+extern gchar *GP_SHADOW_BUF;
+extern gchar *GP_GROUP_BUF;
+extern gchar *GP_GSHADOW_BUF;
+
+
+
 /* Compare strings in lower case */
 int cmplowercase(char *buf, char *cmpstring)
 {
@@ -87,7 +97,7 @@ user_exists(G_CONST_RETURN gchar *username)
     char *check_buffer;
 
     /* Checks if the user exists in passwd */
-    if((fp=fopen(GP_PASSWD,"r"))==NULL)
+    if((fp=fopen(GP_PASSWD_BUF,"r"))==NULL)
     {
 	/* Dont show anything */
     }
@@ -108,7 +118,7 @@ user_exists(G_CONST_RETURN gchar *username)
 		    {
 			sprintf(tempname, check_buffer);
 			tempname[x]='\0';
-			if( ! strcmp(username, tempname) )
+			if( strcmp(username, tempname) == 0 )
 			{
 			    user_exists = 1;
 			    break;
@@ -125,7 +135,7 @@ user_exists(G_CONST_RETURN gchar *username)
 
 
     /* Checks if the user exists in shadow */
-    if((fp=fopen(GP_SHADOW,"r"))==NULL)
+    if((fp=fopen(GP_SHADOW_BUF,"r"))==NULL)
     {
 	/* Dont show anything */
     }
@@ -176,7 +186,7 @@ group_exists(G_CONST_RETURN gchar *groupname)
     char *check_buffer;
 
     /* Checks if the group exists in group */
-    if((fp=fopen(GP_GROUP,"r"))==NULL)
+    if((fp=fopen(GP_GROUP_BUF,"r"))==NULL)
     {
 	/* Dont show anything  */
     }
@@ -196,7 +206,7 @@ group_exists(G_CONST_RETURN gchar *groupname)
 		    {
 			sprintf(tempname, check_buffer);
 			tempname[x]='\0';
-			if( ! strcmp(groupname, tempname) )
+			if( strcmp(groupname, tempname) == 0 )
 			{
 			    group_exists = 1;
 			    break;
@@ -211,7 +221,7 @@ group_exists(G_CONST_RETURN gchar *groupname)
       }
 
     /* Checks if the group exists in gshadow */
-    if((fp = fopen(GP_GSHADOW,"r"))==NULL)
+    if((fp = fopen(GP_GSHADOW_BUF,"r"))==NULL)
     {
 	/* Dont show anything */
     }
@@ -275,7 +285,7 @@ gchar *get_user_setting(gchar *user, gchar *what)
       val_pos = 6;
 
     tempname = allocate(4096);
-    if((fp=fopen(GP_PASSWD,"r"))==NULL)
+    if((fp=fopen(GP_PASSWD_BUF,"r"))==NULL)
     {
         retval = g_strdup_printf(_("User setting not found"));
 	free(tempname);
@@ -295,7 +305,7 @@ gchar *get_user_setting(gchar *user, gchar *what)
             {
                 snprintf(tempname, 4000, "%s", check_buffer);
                 tempname[x]='\0';
-                if( ! strcmp(user, tempname) )
+                if( strcmp(user, tempname) == 0 )
                 {
                     found_user = 1;
                     break;
@@ -380,7 +390,7 @@ gchar *get_group_name(gchar *groupnr)
     val_pos = 2;
 
     temp = allocate(4096);    
-    if((fp=fopen(GP_GROUP,"r"))==NULL)
+    if((fp=fopen(GP_GROUP_BUF,"r"))==NULL)
     {
 	retval = g_strdup_printf(_("Group file not found"));
 	free(temp);
@@ -418,7 +428,7 @@ gchar *get_group_name(gchar *groupnr)
 	    }
 	}
 			
-	if( ! strcmp(groupnr, temp) )
+	if( strcmp(groupnr, temp) == 0 )
 	{
 	    found_group_number = 1;
 	    break;
@@ -461,6 +471,194 @@ gchar *get_group_name(gchar *groupnr)
 }
 
 
+void
+password_virtual_user(G_CONST_RETURN gchar *username, G_CONST_RETURN gchar *password)
+{
+    FILE *fp;
+    long file_size = 0;
+    long i = 0, count = 0;
+    int match = 0;
+    char *line, *tmp, *part, *new_conf;
+    gchar *user_line;
+    const char *encrypted_password;
+    
+    encrypted_password = encrypt_password(password);
+
+    /* Locate the user, get everything but the first parts (username:password:) */
+
+    if((fp=fopen(GP_PASSWD_BUF, "r"))==NULL)
+    {
+       printf("Error reading: %s\n", GP_PASSWD_BUF);
+       return;
+    }
+    fseek(fp, 0, SEEK_END);
+    file_size = ftell(fp);
+    rewind(fp);
+
+    line     = allocate(file_size+1);
+    part     = allocate(file_size+1);
+    tmp      = allocate(file_size+1);
+    new_conf = allocate(file_size+1024);
+
+    if( file_size > 1 )
+    while(fgets(line, file_size, fp)!=NULL)
+    {
+	if( strlen(line) < 10 )
+	  continue;
+	
+	count = 0; match = 0;
+
+	for(i=0; line[i]!='\0'; i++)
+	if( line[i]==':' )
+	{
+	    count++;
+	
+	    /* See if this is the correct username */
+	    if( count == 1 )
+	    {
+		snprintf(tmp, file_size, "%s", line);
+		tmp[i]='\0';
+		
+		
+		/* Do they match */
+		if( strcmp(username, tmp) == 0 )
+		{
+		    match = 1;
+
+		    printf("Usernames Match: %s\n", tmp);
+		}
+		
+		break;
+	    }
+	}
+
+	for(i=i; line[i]!='\0'; i++)
+	if( line[i]==':' )
+	{
+	    count++;
+	
+	    if( match && count == 3 )
+	    {
+		printf("Count and match!\n");
+	    
+		snprintf(part, file_size, "%s", &line[i+1]);
+		break;
+	    }
+	}
+	
+	if( count == 3 && match )
+	{
+	    user_line = g_strdup_printf("%s:%s:%s", username, encrypted_password, part);
+	    strcat(new_conf, user_line);
+	}
+	else
+	  strcat(new_conf, line);
+    }
+    fclose(fp);
+    free(line);
+    free(tmp);
+    free(part);
+
+    /* Write the new and changed virtual password file */
+    if((fp=fopen(GP_PASSWD_BUF, "w+"))==NULL)
+    {
+       printf("Error reading: %s\n", GP_PASSWD_BUF);
+       return;
+    }
+    fputs(new_conf, fp);
+    fclose(fp);
+    free(new_conf);    
+}
+
+
+void
+delete_virtual_user(G_CONST_RETURN gchar *username)
+{
+    FILE *fp;
+    long file_size = 0;
+    long i = 0, count = 0;
+    int match = 0;
+    char *line, *tmp, *part, *new_conf;
+
+    /* Locate the user and remove it */
+    if((fp=fopen(GP_PASSWD_BUF, "r"))==NULL)
+    {
+       printf("Error reading: %s\n", GP_PASSWD_BUF);
+       return;
+    }
+    fseek(fp, 0, SEEK_END);
+    file_size = ftell(fp);
+    rewind(fp);
+
+    line     = allocate(file_size+1);
+    part     = allocate(file_size+1);
+    tmp      = allocate(file_size+1);
+    new_conf = allocate(file_size+1024);
+
+    if( file_size > 1 )
+    while(fgets(line, file_size, fp)!=NULL)
+    {
+	if( strlen(line) < 10 )
+	  continue;
+	
+	count = 0; match = 0;
+
+	for(i=0; line[i]!='\0'; i++)
+	if( line[i]==':' )
+	{
+	    count++;
+	
+	    /* See if this is the correct username */
+	    if( count == 1 )
+	    {
+		snprintf(tmp, file_size, "%s", line);
+		tmp[i]='\0';
+		
+		/* Do they match */
+		if( strcmp(username, tmp) == 0 )
+		{
+		    match = 1;
+		}
+		break;
+	    }
+	}
+
+	for(i=i; line[i]!='\0'; i++)
+	if( line[i]==':' )
+	{
+	    count++;
+	
+	    if( match && count == 3 )
+	    {
+		snprintf(part, file_size, "%s", &line[i+1]);
+		break;
+	    }
+	}
+	
+	if( count == 3 && match )
+	{
+	    /* Skip this user line */
+	}
+	else
+	  strcat(new_conf, line);
+    }
+    fclose(fp);
+    free(line);
+    free(tmp);
+    free(part);
+
+    /* Write the new and changed virtual password file */
+    if((fp=fopen(GP_PASSWD_BUF, "w+"))==NULL)
+    {
+       printf("Error reading: %s\n", GP_PASSWD_BUF);
+       return;
+    }
+    fputs(new_conf, fp);
+    fclose(fp);
+    free(new_conf);    
+}
+
+
 void
 password_user(G_CONST_RETURN gchar *username, G_CONST_RETURN gchar *password)
 {
@@ -612,7 +810,7 @@ int is_banned(char *user)
 		if( user_buffer[i]=='\n' || user_buffer[i]=='\r' )
 		  user_buffer[i]='\0';
 	    }
-	    if( ! strcmp(user_buffer, user) )
+	    if( strcmp(user_buffer, user) == 0 )
 	    {
 		ret = 1;
 		break;
