@@ -150,7 +150,6 @@ void generate_cert(struct w *widgets)
     }
 
     config = g_strconcat("\n",
-//    "dir = /etc/gadmin-proftpd/certs\n",
     "dir = ", cert_dir, "\n",
 
     "[ ca ]\n",
@@ -187,9 +186,9 @@ void generate_cert(struct w *widgets)
     "req_extensions = v3_req\n\n",
 
     "[ req_distinguished_name ]\n",
-    "0.organizationName = Organization name\n",
-    "organizationalUnitName = Organization unit\n",
-    "emailAddress = email@example.org\n",
+    "0.organizationName = ", org, "\n",
+    "organizationalUnitName = ", org_unit, "\n",
+    "emailAddress = ", email, "\n",
     "emailAddress_max = 80\n",
     "localityName = ", city, "\n",
     "stateOrProvinceName = ", state, "\n",
@@ -231,6 +230,7 @@ void generate_cert(struct w *widgets)
 	return;
     }
     fclose(fp);
+
     /* Secure the password file. */
 /*    cmd = g_strdup_printf("chmod 750 %s", passfile);
     if( ! run_command(cmd) )
@@ -245,6 +245,7 @@ void generate_cert(struct w *widgets)
     }
     g_free(cmd);
 */
+
     if((fp=fopen(passfile, "w"))==NULL)
     {
 	info = g_strdup_printf(_("Error writing openssl password file here:\n%s\n"), passfile);
