@@ -380,7 +380,7 @@ main(int argc, char *argv[])
 	row = 0;
 	while( row < max_rows )
 	{
-	    if( ! strcmp(userlist[row].user, user) )
+	    if( strcmp(userlist[row].user, user) == 0 )
 	    {
     	        /* Upload */
     	        if( strstr(action, "i") )
@@ -608,7 +608,7 @@ main(int argc, char *argv[])
 		/* Minimize writing to the same file */
 		if( row > 0 && row < max_rows-1 )
 		{
-		    if( ! strcmp(userlist[row].dir, userlist[row-1].dir) )
+		    if( strcmp(userlist[row].dir, userlist[row-1].dir) == 0 )
 		    {
 			row++;
 			continue;
