@@ -174,7 +174,7 @@ void add_import_users(GtkTreeModel *model, GtkTreePath *path,
 
 
         if( strstr("Virtualhost", (char *)global_server_type) && ! found_server
-	&&  ! strcmp(old_buffer, address_buffer) ) 
+	&&  strcmp(old_buffer, address_buffer) == 0 ) 
 	{
 	    /* Lets see if its the same port as the selected one */
 	    /* If this server has the same port its the correct server */
@@ -192,7 +192,7 @@ void add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	    
 		/* This will expect the servers port on the second line ! 
 		 * else itll miss some vaules .. */
-		if( strstr(old_buffer, "Port") && ! strcmp(old_buffer, port_buffer) )
+		if( strstr(old_buffer, "Port") && strcmp(old_buffer, port_buffer) == 0 )
 		{
 		    found_server = 1;
 		    break;
@@ -231,7 +231,7 @@ void add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	}
 
 	/* Does this user exist in this server... */
-	if( ! strcmp(old_buffer, user_check) && ! strstr(old_buffer, "AllowUser") )
+	if( strcmp(old_buffer, user_check) == 0 && ! strstr(old_buffer, "AllowUser") )
 	{
 	    info = g_strdup_printf(_("The user: \"%s\" was not imported\nbecause it already existed in this server.\n"), username);
 	    show_info(info);
@@ -348,7 +348,7 @@ void add_import_users(GtkTreeModel *model, GtkTreePath *path,
 	strcat(new_buffer, old_buffer);
 
         if( strstr("Virtualhost", (char *)global_server_type) && ! found_server
-	&&  ! strcmp(old_buffer, address_buffer) )
+	&&  strcmp(old_buffer, address_buffer) == 0 )
 	{
 	    /* Lets see if its the same port as the selected one */
 	    /* If this server has the same port its the correct server. */
@@ -361,7 +361,7 @@ void add_import_users(GtkTreeModel *model, GtkTreePath *path,
 
 		/* This will expect the servers port on the second line ! 
 		 * else itll miss some values .. */
-		if( strstr(old_buffer, "Port") && ! strcmp(old_buffer, port_buffer) )
+		if( strstr(old_buffer, "Port") && strcmp(old_buffer, port_buffer) == 0 )
 		{
 		    found_server = 1;
 		    break;
