@@ -130,7 +130,7 @@ niutil_user_exists(G_CONST_RETURN gchar *username)
 		    {
 			sprintf(tempname, check_buffer);
 			tempname[x]='\0';
-			if( ! strcmp(username, tempname) )
+			if( strcmp(username, tempname) == 0 )
 			{
 			    user_exists=1;
 			    break;
