@@ -32,6 +32,7 @@
 #include "functions.h"
 #include "get_option_pos.h"
 
+extern gulong virtual_users_toggled_signal;
 
 extern char global_server_address[1024];
 extern char global_server_port[1024];
@@ -41,16 +42,22 @@ extern int use_tls;
 extern int use_ratio;
 extern int use_quota;
 
+extern gchar *GP_PASSWD_BUF;
+extern gchar *GP_SHADOW_BUF;
+extern gchar *GP_GROUP_BUF;
+extern gchar *GP_GSHADOW_BUF;
+
 
 
 void populate_server_settings(struct w *widgets)
 {
     /* Lists the values for the selected server in the server tab */
     FILE *fp;
-    int x = 0, val = 0, once = 0, found = 0;
+    int x = 0, val = 0, found = 0;
+    int found_maskaddr = 0;
     long conf_size;
     char *old_buffer, *port_buffer, *new_buffer, *address_buffer, *openssl_conf;
-    gchar *settings_conf, *info = NULL, *utf8 = NULL;
+    gchar *settings_conf, *path, *utf8 = NULL;
 
     if((fp=fopen(PROFTPD_CONF, "r"))==NULL)
     {
@@ -60,8 +67,8 @@ void populate_server_settings(struct w *widgets)
     conf_size = ftell(fp);
     rewind(fp);
 
-    old_buffer = allocate(conf_size);
-    new_buffer = allocate(conf_size);
+    old_buffer = allocate(conf_size+1);
+    new_buffer = allocate(conf_size+1);
     address_buffer = allocate(8192+15);
 
 
@@ -70,12 +77,12 @@ void populate_server_settings(struct w *widgets)
     else
        sprintf(address_buffer, global_server_address);
      
-      port_buffer = allocate(8192+3);               
+    port_buffer = allocate(8192+3);               
 
-      sprintf(port_buffer, "Port %s", global_server_port);
+    sprintf(port_buffer, "Port %s", global_server_port);
 
-      if( ! strstr((char *)global_server_type, "Virtualhost") )
-        found = 1;
+    if( ! strstr((char *)global_server_type, "Virtualhost") )
+      found = 1;
 
     if( conf_size > 1 )
     while(fgets(old_buffer, conf_size, fp)!=NULL)
@@ -174,7 +181,8 @@ void populate_server_settings(struct w *widgets)
 	    }
 	    x++;
 	    if( old_buffer[x]!='\0' )
-	      sprintf(new_buffer, &old_buffer[x]);
+	      snprintf(new_buffer, 1000, "%s", &old_buffer[x]);
+
 	    for(x=0; new_buffer[x]!='\0'; x++)
 	    {
 	        if( new_buffer[x]=='"' || new_buffer[x]=='\n' )
@@ -183,6 +191,7 @@ void populate_server_settings(struct w *widgets)
 		    break;
 		}
 	    }
+	    
 	    utf8 = g_locale_to_utf8(new_buffer, strlen(new_buffer), NULL, NULL, NULL);
     	    gtk_entry_set_text(GTK_ENTRY(widgets->server_set_entry[1]), utf8);
 
@@ -204,15 +213,17 @@ void populate_server_settings(struct w *widgets)
 		gtk_entry_set_text(GTK_ENTRY(widgets->server_set_entry[2]), utf8);
 	    }
 
-	    if( ! strstr(old_buffer, "#") )
-	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->server_set_combo[1]), 0);
-	    else
+	    if( strstr(old_buffer, "#") )
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->server_set_combo[1]), 1);
+	    else
+	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->server_set_combo[1]), 0);
+
+	    found_maskaddr = 1;
 	}
 
    	if( cmplowercase(old_buffer, "serveradmin") )
 	{   
-	    sprintf(new_buffer, &old_buffer[12]);
+	    snprintf(new_buffer, 1000, "%s", &old_buffer[12]);
 	    if( new_buffer[strlen(new_buffer)-1]=='\n' || new_buffer[strlen(new_buffer)-1]=='\r' )
 	      new_buffer[strlen(new_buffer)-1]='\0';
 
@@ -471,7 +482,7 @@ void populate_server_settings(struct w *widgets)
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->server_set_combo[12]), 1);
 	}
 
-	if( cmplowercase(old_buffer, "systemlog") )
+	if( cmplowercase(old_buffer, "systemlog") && strlen(old_buffer) > 9 )
 	{
 	    sprintf(new_buffer, &old_buffer[10]);
 	    if( new_buffer[strlen(new_buffer)-1]=='\n' || new_buffer[strlen(new_buffer)-1]=='\r' )
@@ -573,9 +584,6 @@ void populate_server_settings(struct w *widgets)
     free(port_buffer);
 
 
-    gtk_widget_show_all(widgets->main_window);
-
-
     /* Add settings.conf values to the server settings. */
     settings_conf = g_strdup_printf("%s/settings.conf", GP_APPCONFDIR);
     if((fp=fopen(settings_conf, "r"))==NULL)
@@ -587,8 +595,8 @@ void populate_server_settings(struct w *widgets)
     conf_size = ftell(fp);
     rewind(fp);
 
-    old_buffer = allocate(conf_size);
-    new_buffer = allocate(conf_size);
+    old_buffer = allocate(conf_size+1);
+    new_buffer = allocate(conf_size+1);
 
     if( conf_size > 1 )
     while(fgets(old_buffer, conf_size, fp)!=NULL)
@@ -634,41 +642,6 @@ void populate_server_settings(struct w *widgets)
 	    else
 	      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->server_set_combo[7]), 1);
 	}
-
-/*
-This will be deprecated in a few years, the default conf adds gp_useradd_homedir_path (changed 2005-08-24)
-	if( cmplowercase(old_buffer, "useradd_root_path") )
-	{
-	    sprintf(new_buffer, &old_buffer[22]);
-	    if( new_buffer[strlen(new_buffer)-1]=='\n' || new_buffer[strlen(new_buffer)-1]=='\r' )
-	      new_buffer[strlen(new_buffer)-1]='\0';
-
-	    utf8 = g_locale_to_utf8(new_buffer, strlen(new_buffer), NULL, NULL, NULL);
-            gtk_entry_set_text(GTK_ENTRY(widgets->server_set_entry[4]), utf8);
-	}
-*/
-/*
-	if( cmplowercase(old_buffer, "useradd_homedir_path") )
-	{
-	    sprintf(new_buffer, &old_buffer[25]);
-	    if( new_buffer[strlen(new_buffer)-1]=='\n' || new_buffer[strlen(new_buffer)-1]=='\r' )
-	      new_buffer[strlen(new_buffer)-1]='\0';
-
-	    utf8 = g_locale_to_utf8(new_buffer, strlen(new_buffer), NULL, NULL, NULL);
-            gtk_entry_set_text(GTK_ENTRY(widgets->server_set_entry[4]), utf8);
-	}
-
-	if( cmplowercase(old_buffer, "useradd_upload_path") )
-	{
-	    sprintf(new_buffer, &old_buffer[24]);
-	    if( new_buffer[strlen(new_buffer)-1]=='\n' || new_buffer[strlen(new_buffer)-1]=='\r' )
-	      new_buffer[strlen(new_buffer)-1]='\0';
-
-	    utf8 = g_locale_to_utf8(new_buffer, strlen(new_buffer), NULL, NULL, NULL);
-            gtk_entry_set_text(GTK_ENTRY(widgets->server_set_entry[5]), utf8);
-	}
-*/          // Generate to path [Entry] [Button] <- Yes
-
 	if( cmplowercase(old_buffer, "html_path") )
 	{
 	    sscanf(old_buffer, "%*s %s", new_buffer);
@@ -678,16 +651,52 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    utf8 = g_locale_to_utf8(new_buffer, strlen(new_buffer), NULL, NULL, NULL);
             gtk_entry_set_text(GTK_ENTRY(widgets->server_set_entry[6]), utf8);
 	}
+	/* Virtual users: If "on" switch to virtual users and groups else system users and groups */
+	if( cmplowercase(old_buffer, "virtual_users") )
+	{
+	    /* Free paths first */
+	    g_free(GP_PASSWD_BUF);
+	    g_free(GP_SHADOW_BUF);
+	    g_free(GP_GROUP_BUF);
+	    g_free(GP_GSHADOW_BUF);
 
+
+	    /* Block the togglebuttons signal */
+	    g_signal_handler_block(GTK_WIDGET(widgets->server_set_check_virtual), virtual_users_toggled_signal);
+	
+	    if( cmplowercase(old_buffer, "on") )
+	    {
+	        gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(widgets->server_set_check_virtual), TRUE);
+	    
+		path = g_strdup_printf("%s/users", GP_APPCONFDIR);
+		make_dir_chmod(path, "0700");
+		g_free(path);
+
+		GP_PASSWD_BUF  = g_strdup_printf("%s/users/proftpd.passwd", GP_APPCONFDIR);
+		GP_GROUP_BUF   = g_strdup_printf("%s/users/proftpd.group", GP_APPCONFDIR);
+
+		GP_SHADOW_BUF  = g_strdup_printf("%s/users/shadow_NOT_USED", GP_APPCONFDIR);
+		GP_GSHADOW_BUF = g_strdup_printf("%s/users/gshadow_NOT_USED", GP_APPCONFDIR);
+	    }
+	    else
+	    {
+	        gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(widgets->server_set_check_virtual), FALSE);
+	    
+		GP_PASSWD_BUF  = g_strdup_printf("%s", GP_PASSWD);
+		GP_SHADOW_BUF  = g_strdup_printf("%s", GP_SHADOW);
+		GP_GROUP_BUF   = g_strdup_printf("%s", GP_GROUP);
+		GP_GSHADOW_BUF = g_strdup_printf("%s", GP_GSHADOW);
+	    }
+	    
+	    /* Unblock the togglebuttons signal*/
+	    g_signal_handler_unblock(GTK_WIDGET(widgets->server_set_check_virtual), virtual_users_toggled_signal);
+	}
     }
     fclose(fp);
+    free(old_buffer);
+    free(new_buffer);
     g_free(settings_conf);
 
-    if( utf8!=NULL )
-      g_free(utf8);
-
-    gtk_widget_show_all(widgets->main_window);
-
 
     /* If we have the TLS module then list the values for it from gadmin-proftpd-openssl.conf. */
     /* These are generated by gadmin-proftpd so theres no need to use cmplowercase */
@@ -696,16 +705,15 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	openssl_conf = g_strdup_printf("%s/certs/gadmin-proftpd-openssl.conf", GP_APPCONFDIR);
 	if((fp=fopen(openssl_conf, "r"))==NULL)
 	{
-	
+	    g_free(openssl_conf);	
 	    return;
 	}
-	
 	fseek(fp, 0, SEEK_END);
 	conf_size = ftell(fp);
 	rewind(fp);
 
-	old_buffer = allocate(conf_size);
-	new_buffer = allocate(conf_size);
+	old_buffer = allocate(conf_size+1);
+	new_buffer = allocate(conf_size+1);
 	
 	if( conf_size > 1 )
 	while(fgets(old_buffer, conf_size, fp)!=NULL)
@@ -713,9 +721,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* Server address */
 	    if( strstr(old_buffer, "commonName = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -728,9 +736,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* Email address */
 	    if( strstr(old_buffer, "emailAddress = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -743,9 +751,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* State or province */
 	    if( strstr(old_buffer, "stateOrProvinceName = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -758,9 +766,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* City or town */
 	    if( strstr(old_buffer, "localityName = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -773,9 +781,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* Organization */
 	    if( strstr(old_buffer, "organizationName = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -788,9 +796,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* Organizational unit */
 	    if( strstr(old_buffer, "organizationalUnitName = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -803,9 +811,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* Default certificate bits */
 	    if( strstr(old_buffer, "default_bits = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-    		    if( old_buffer[x-2]=='=' )
+    		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -821,9 +829,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* Default days valid */
     	    if( strstr(old_buffer, "default_days = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -839,9 +847,9 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 	    /* Country code */
 	    if( strstr(old_buffer, "countryName = ") )
 	    {
-		for(x=0; old_buffer[x]; x++)
+		for(x=0; old_buffer[x]!='\0'; x++)
 		{
-		    if( old_buffer[x-2]=='=' )
+		    if( x >= 2 && old_buffer[x-2]=='=' )
 		    {
 			snprintf(new_buffer, old_buffer[x]+x, old_buffer+x);
 			new_buffer[strlen(new_buffer)-1]='\0';
@@ -852,13 +860,16 @@ This will be deprecated in a few years, the default conf adds gp_useradd_homedir
 		}
 	    }
 	}
+	fclose(fp);
 	g_free(openssl_conf);
 	free(new_buffer);
 	free(old_buffer);
-	fclose(fp);
     }
+
     if( utf8!=NULL )
       g_free(utf8);
 
-    gtk_widget_show_all(widgets->main_window);
+    /* Set MasqueradeAddress to off if not found */
+    if( ! found_maskaddr )
+      gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->server_set_combo[1]), 1);
 }
