@@ -175,9 +175,6 @@ void populate_user_settings(struct w *widgets)
 	/* Comment */
 	gtk_entry_set_text(GTK_ENTRY(widgets->user_set_entry[3]), "");
 
-	/* Home directory */
-// Removed	gtk_entry_set_text(GTK_ENTRY(widgets->user_set_entry[4]), "/var/ftp");
-
 	/* Shell */
 	gtk_combo_box_set_active(GTK_COMBO_BOX(widgets->user_set_combo[0]), 0);
 
@@ -223,8 +220,8 @@ void populate_user_settings(struct w *widgets)
     conf_size = ftell(fp);
     rewind(fp);
 
-    old_buffer = allocate(conf_size);
-    new_buffer = allocate(conf_size);   
+    old_buffer = allocate(conf_size+1);
+    new_buffer = allocate(conf_size+1);
     user_buffer = allocate(4096);
     address_buffer = allocate(8192);
     port_buffer = allocate(8192);
@@ -256,7 +253,7 @@ void populate_user_settings(struct w *widgets)
 	while(fgets(old_buffer, conf_size, fp)!=NULL)
 	{
 	    /* If its the correct address */
-	    if( ! strcmp(old_buffer, address_buffer) )
+	    if( strcmp(old_buffer, address_buffer) == 0 )
 	    {
 	       while(fgets(old_buffer, conf_size, fp)!=NULL)
 	       {
@@ -264,7 +261,7 @@ void populate_user_settings(struct w *widgets)
 		  if( strstr(old_buffer, "</VirtualHost>") )
 		    break;
 		 
-		  if( ! strcmp(old_buffer, port_buffer) )
+		  if( strcmp(old_buffer, port_buffer) == 0 )
 		  {
 		     found = 1;
 	             break;
@@ -303,19 +300,15 @@ void populate_user_settings(struct w *widgets)
 
 
 	/* We have found the correct user .. insert the settings */
-	if( ! strcmp(old_buffer, user_buffer) && ! strstr(old_buffer, "AllowUser")
+	if( strcmp(old_buffer, user_buffer) == 0 && ! strstr(old_buffer, "AllowUser")
         &&  ! strstr(old_buffer, "DenyUser") && ! strstr(old_buffer, "FakeUser") )
 	{
-
 		/* Insert the root directory to the directory treeview (collected in the loop above) */
 		new_buffer[strlen(new_buffer)-2]='\0';
 		utf8 = g_locale_to_utf8(&new_buffer[11], strlen(&new_buffer[11]), NULL, NULL, NULL);
 		gtk_list_store_append(GTK_LIST_STORE(widgets->directory_store), &iter);
 		gtk_list_store_set(GTK_LIST_STORE(widgets->directory_store), &iter, 0, utf8, -1);
 
-		/* Insert the root directory into the entry */
-// Removed		gtk_entry_set_text(GTK_ENTRY(widgets->user_set_entry[4]), utf8);
-
 		/* Username on the line after <Anonymous /var/ftp/dir> (cant be set before root dir) */
 		sscanf(old_buffer, "%*s %s", new_buffer);
 		utf8 = g_locale_to_utf8(new_buffer, strlen(new_buffer), NULL, NULL, NULL);
@@ -348,7 +341,7 @@ void populate_user_settings(struct w *widgets)
         		strcpy(new_buffer, old_buffer);
 			for(i=0; new_buffer[i]!='\0'; i++)
 			{
-			    if( new_buffer[i-2]=='m' )
+			    if( i >= 2 && new_buffer[i-2]=='m' )
 			    {
 				new_buffer[strlen(new_buffer)-1]='\0';
 				utf8 = g_locale_to_utf8(&new_buffer[i], strlen(&new_buffer[i]), NULL, NULL, NULL);
@@ -467,7 +460,7 @@ void populate_user_settings(struct w *widgets)
 			for(i=0; new_buffer[i]!='\0'; i++)
 			{
 	    		    /* "y " */
-			    if( new_buffer[i-2]=='y' )
+			    if( i >= 2 && new_buffer[i-2]=='y' )
 			    {
 				new_buffer[strlen(new_buffer)-2]='\0';
 				utf8 = g_locale_to_utf8(&new_buffer[i], strlen(&new_buffer[i]), NULL, NULL, NULL);
@@ -577,16 +570,18 @@ void populate_user_settings(struct w *widgets)
                     gtk_tree_model_get(GTK_TREE_MODEL(shell_store), &shell_iter, 0, &shell, -1);
                     if( shell )
                     {
-                        if( ! strcmp(val, shell) )
+                        if( strcmp(val, shell) == 0 )
                         {
                             gtk_combo_box_set_active_iter(GTK_COMBO_BOX(widgets->user_set_combo[0]), &shell_iter);
                             break;
                         }
+			g_free(shell);
                     }
                }    while(gtk_tree_model_iter_next(GTK_TREE_MODEL(shell_store), &shell_iter));
 	}
     }
     g_free(val);
+    g_free(username);
 
     if( utf8!=NULL )
       g_free(utf8);
