@@ -217,8 +217,8 @@ void add_standard_conf(struct w *widgets)
     "DefaultServer on\n",
     "Umask 022\n",
     "ServerName \"0.0.0.0\"\n",
-    "ServerIdent on \"My FTPD\"\n",
-    "ServerAdmin Admin@this.domain.topdomain\n",
+    "ServerIdent on \"My FTP Server\"\n",
+    "ServerAdmin email@example.org\n",
     "IdentLookups off\n",
     "UseReverseDNS off\n",
     "Port 21\n",
@@ -232,13 +232,7 @@ void add_standard_conf(struct w *widgets)
     "TimeoutIdle 120\n",
     "DisplayLogin ", WELCOME_MESSAGE, "\n",
 
-/*
-    Fedora back ported/patched their version. Hmmz...
-    if( strstr(global_version,"1.2.") || strstr(global_version, "1.3.0") )
-      "DisplayFirstChdir .message\n"
-    else
-      "DisplayChdir .message\n"
-*/
+    "DisplayChdir .message\n"
 
     "User ", SERVER_USER, "\n",
     "Group ", SERVER_GROUP, "\n",
@@ -249,20 +243,12 @@ void add_standard_conf(struct w *widgets)
     "AllowRetrieveRestart on\n",
     "AllowStoreRestart on\n",
     "DeleteAbortedStores off\n",
-    "TransferRate RETR 30\n",
-    "TransferRate STOR 40\n",
-    "TransferRate STOU 40\n",
-    "TransferRate APPE 40\n",
+    "TransferRate RETR 220\n",
+    "TransferRate STOR 250\n",
+    "TransferRate STOU 250\n",
+    "TransferRate APPE 250\n",
     "SystemLog ", GP_VARDIR, "/log/secure\n",
     "RequireValidShell off\n",
-
-    "#gp_random_username_length 6\n",
-    "#gp_random_password_length 6\n",
-    "#gp_randomize_case lower\n",
-    "#gp_useradd_homedir_path /var/ftp\n",
-    "#gp_html_path ", HTML_STATISTICS, "\n",
-    "#gp_welcome_name ", WELCOME_MESSAGE, "\n\n",
-
     "<IfModule mod_tls.c>\n",
     "TLSEngine off\n",
     "TLSRequired off\n",
