@@ -100,32 +100,38 @@ int status_update(struct w *widgets)
 {
     /* Status update for ftpwho, total transfer rates and activated/deactivated status. */
     FILE *fp;
-    GtkTextBuffer *transfer_textbuffer;
-    GtkTextMark *mark;
-    gchar *mark_name=NULL;
-    GtkTextIter transfer_iter;
-    long in_val=0, out_val=0, inxfer=0, outxfer=0, totalxfer=0;
+    GtkTreeIter iter;
+    long i=0, in_val=0, out_val=0, inxfer=0, outxfer=0, totalxfer=0;
     int incoming=0, outgoing=0;
-    char *read_buffer, *get_buffer;
+    char *line, *tmp;
     gchar *info, *path, *cmd, *utf8=NULL;
 
     char *pid;
     pid = get_process_pid("proftpd", "accepting connections");
 
-    if( ! strcmp(pid, "0") )
+    if( strcmp(pid, "0") == 0 )
     {
 	activated = 0;
 	info = g_strdup_printf(_("Status: Deactivated"));
 	utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
 	gtk_label_set_text(GTK_LABEL(widgets->status_label), utf8);
 	g_free(info);
+	if( utf8!=NULL )
+	  g_free(utf8);
 
 	/* Show non ftp usage */    
         info = g_strdup_printf(_("The server is deactivated.\n"));
 	utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
-	transfer_textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widgets->transfer_textview));
-	gtk_text_buffer_set_text(transfer_textbuffer, utf8, strlen(utf8));
+
+
+//	transfer_textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widgets->transfer_textview));
+//	gtk_text_buffer_set_text(transfer_textbuffer, utf8, strlen(utf8));
+
+
 	g_free(info);
+	if( utf8!=NULL )
+	  g_free(utf8);
+
 	return (TRUE);
     }
     else
@@ -136,13 +142,15 @@ int status_update(struct w *widgets)
     	  utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
     	  gtk_label_set_text(GTK_LABEL(widgets->status_label), utf8);
 	  g_free(info);
+	  if( utf8!=NULL )
+	    g_free(utf8);
       }
     free(pid);
 
 
 
-    /* The server is activated, insert ftpwho information in the textview */
-    get_buffer = allocate(MAX_READ_POPEN+1);
+    /* The server is activated, insert ftpwho information in the treeview */
+
     
     /* Add the scoreboard file if its missing to avoid useless message */
     path = g_strdup_printf("%s/%s", GP_VARDIR, "run/proftpd");
@@ -179,36 +187,138 @@ int status_update(struct w *widgets)
 
 
     /* FTP who update and total xferrates */
+    
+    gtk_list_store_clear(GTK_LIST_STORE(widgets->transfer_store));
+    
+    
     cmd = g_strdup_printf("%s -v", FTPWHO_BINARY);
     if((fp=popen(cmd, "r"))==NULL)
     {
-        g_free(cmd);
 
 	/* Show ftpwho error information in the textview */    
         info = g_strdup_printf(_("Error running ftpwho -v.\n"));
 	utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
-	transfer_textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widgets->transfer_textview));
-	gtk_text_buffer_set_text(transfer_textbuffer, utf8, strlen(utf8));
+
+
+//	transfer_textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widgets->transfer_textview));
+//	gtk_text_buffer_set_text(transfer_textbuffer, utf8, strlen(utf8));
+
+
+        g_free(cmd);
 	g_free(info);
+	if( utf8!=NULL )
+	  g_free(utf8);
+	
         return(TRUE);
     }
     g_free(cmd);
     fflush(fp);
 
-    read_buffer = allocate(MAX_READ_POPEN+1);
-    while(fgets(read_buffer, MAX_READ_POPEN, fp)!=NULL)
+    line = allocate(MAX_READ_POPEN+1);
+    tmp  = allocate(MAX_READ_POPEN+20000); /* +20000 for translations and filenames */
+
+    while(fgets(line, MAX_READ_POPEN, fp)!=NULL)
     {	
-        strcat(get_buffer, read_buffer);
+	if( line==NULL || strlen(line) < 10
+	|| strstr(line, "FTP daemon [")  || strstr(line, "up for")
+	|| strstr(line, "Service class") || strstr(line, "no users connected") )
+	  continue;
+
+	if( strstr(line, " idle") /* idle is located at the end of the line */ 
+	||  strstr(line, " STOR ") 
+	||  strstr(line, " STOU ") 
+	||  strstr(line, " APPE ") 
+	||  strstr(line, " DELE ") 
+	||  strstr(line, " RETR ") )
+	{
+	    gtk_list_store_append(GTK_LIST_STORE(widgets->transfer_store), &iter);
+	
+	    /* The line looks like this:
+	       7154 test [ 0m27s] (n/a) APPE 05 - Soundfile.ogg */
+	    
+	    /* Insert PID */
+	    sscanf(line, "%s", tmp);
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 0, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+
+	    /* Insert Username */
+	    sscanf(line, "%*s %s", tmp);
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 1, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+
+	    /* Insert Action */
+	    if( strstr(line, " idle") )
+	      snprintf(tmp, 100, "%s", _("Idle"));
+	    if( strstr(line, " APPE ") || strstr(line, " STOR ") || strstr(line, " STOU ") )
+	      snprintf(tmp, 100, "%s", _("Uploading"));
+	    if( strstr(line, " RETR ") )
+	      snprintf(tmp, 100, "%s", _("Downloading"));
+	    if( strstr(line, " DELE ") )
+	      snprintf(tmp, 100, "%s", _("Deleting"));
+
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 2, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+
+	    /* Insert filename and handle split filenames */
+	    if( ! strstr(line, " idle") )
+	    for(i=0; line[i]!='\0'; i++)
+	    {
+		if( line[i]=='A' && line[i+1]=='P' && line[i+2]=='P' && line[i+3]=='E'  )
+	    	{
+		    snprintf(tmp, 16384, "%s", &line[i+5]);
+		    break;
+		}
+		if( line[i]=='S' && line[i+1]=='T' && line[i+2]=='O' && line[i+3]=='R'  )
+	    	{
+		    snprintf(tmp, 16384, "%s", &line[i+5]);
+		    break;
+		}
+		if( line[i]=='S' && line[i+1]=='T' && line[i+2]=='O' && line[i+3]=='U'  )
+	    	{
+		    snprintf(tmp, 16384, "%s", &line[i+5]);
+		    break;
+		}
+		if( line[i]=='R' && line[i+1]=='E' && line[i+2]=='T' && line[i+3]=='R'  )
+	    	{
+		    snprintf(tmp, 16384, "%s", &line[i+5]);
+		    break;
+		}
+		if( line[i]=='D' && line[i+1]=='E' && line[i+2]=='L' && line[i+3]=='E'  )
+	    	{
+		    snprintf(tmp, 16384, "%s", &line[i+5]);
+		    break;
+		}
+	    }
+	    if( strstr(line, " idle") )
+	      snprintf(tmp, 16384, "%s", "None");
+
+	    if( tmp[strlen(tmp)-1]=='\n' )
+	        tmp[strlen(tmp)-1]='\0';
+	    
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 5, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+
         /* Total Xferrates incoming/outgoing/total bandwidth usage */
         /* Assumes that the ftpwho output shows stor retr appe and on the next line KB/s */
 		
         /* Incoming transfers */
-        if( strstr(read_buffer, "KB/s: ") && incoming == 1 
-        && ! strstr(read_buffer, "inf") && ! strstr(read_buffer, "nan") )
+        if(  strstr(line, "KB/s: ") && incoming == 1 
+        && ! strstr(line, "inf") && ! strstr(line, "nan") )
         {
-	    if( chars_are_digits(&read_buffer[7]) )
-	      sscanf(read_buffer, "%*s %li", &in_val);
-
+	    if( chars_are_digits(&line[7]) )
+	    {
+	        sscanf(line, "%*s %li", &in_val);
+	    }
+	    
 	    if( in_val > 0 )
 	    {
 		inxfer = inxfer+in_val;
@@ -216,18 +326,57 @@ int status_update(struct w *widgets)
 	    }
 	}
 
-	if( strstr(read_buffer, " STOR ") || strstr(read_buffer, " STOU ")
-	|| strstr(read_buffer, " APPE ") )
+	if( strstr(line, "KB/s: ") && ! strstr(line, "inf") && ! strstr(line, "nan") )
+	{
+	    /* Insert KB/s */
+	    snprintf(tmp, 100, "%s", &line[7]);
+	    if( tmp[strlen(tmp)-1]=='\n' )
+        	tmp[strlen(tmp)-1]='\0';
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 3, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+	if( strstr(line, "client: ") )
+	{
+	    /* Insert Client address */
+	    sscanf(line, "%*s %s", tmp);
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 6, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+	if( strstr(line, "server: ") )
+	{
+	    /* Insert Server address */
+	    sscanf(line, "%*s %s", tmp);
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 7, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+	if( strstr(line, "location: ") )
+	{
+	    /* Insert directory location */
+	    sscanf(line, "%*s %s", tmp);
+	    utf8 = g_locale_to_utf8(tmp, strlen(tmp), NULL, NULL, NULL);
+	    gtk_list_store_set(GTK_LIST_STORE(widgets->transfer_store), &iter, 4, utf8, -1);
+	    if( utf8!=NULL )
+	      g_free(utf8);
+	}
+
+
+	if( strstr(line, " STOR ") || strstr(line, " STOU ") || strstr(line, " APPE ") )
 	  incoming++;
 	else
 	  incoming = 0;
 	
 	/* Outgoing transfers */
-	if( strstr(read_buffer, "KB/s: ") && outgoing == 1 
-	&& ! strstr(read_buffer, "inf") && ! strstr(read_buffer, "nan") )
+	if(  strstr(line, "KB/s: ") && outgoing == 1 
+	&& ! strstr(line, "inf") && ! strstr(line, "nan") )
 	{
-	    if( chars_are_digits(&read_buffer[7]) )
-	      sscanf(read_buffer, "%*s %li", &out_val);
+	    if( chars_are_digits(&line[7]) )
+	      sscanf(line, "%*s %li", &out_val);
 
 	    if( out_val > 0 )
 	    {
@@ -236,60 +385,54 @@ int status_update(struct w *widgets)
 	    }
 	}
 
-	if( strstr(read_buffer, " RETR ") )
+	if( strstr(line, " RETR ") )
 	  outgoing++;
 	else
 	  outgoing = 0;
     }
     pclose(fp);
-    free(read_buffer);
+    free(line);
+    free(tmp);
 
     
     /* Total bandwidth usage */
     if( totalxfer >= 0 )
     {
-	info = g_strdup_printf("%li %s", totalxfer, "KB/s");
+	info = g_strdup_printf("\t\t %s %li", _(" Total KB/s: "), totalxfer);
         utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
-	gtk_entry_set_text(GTK_ENTRY(widgets->total_bandwidth_entry), utf8);
+
+	gtk_label_set_text(GTK_LABEL(widgets->total_bandwidth_label), utf8);
+
 	g_free(info);
+	if( utf8!=NULL )
+	  g_free(utf8);
     }
 
     /* Total incoming xfers */
     if( inxfer >= 0 )
     {    
-        info = g_strdup_printf("%li %s", inxfer, "KB/s");
+        info = g_strdup_printf("\t\t\t %s %li", _(" Incoming KB/s: "), inxfer);
         utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
-        gtk_entry_set_text(GTK_ENTRY(widgets->total_incoming_entry), utf8);
+
+	gtk_label_set_text(GTK_LABEL(widgets->total_incoming_label), utf8);
+
         g_free(info);
+	if( utf8!=NULL )
+	  g_free(utf8);
     }
 
     /* Total outgoing xfers */
     if( outxfer >= 0 )
     {
-        info = g_strdup_printf("%li %s", outxfer, "KB/s");
+        info = g_strdup_printf("\t\t\t %s %li", _(" Outgoing KB/s: "), outxfer);
         utf8 = g_locale_to_utf8(info, strlen(info), NULL, NULL, NULL);
-        gtk_entry_set_text(GTK_ENTRY(widgets->total_outgoing_entry), utf8);
+
+	gtk_label_set_text(GTK_LABEL(widgets->total_outgoing_label), utf8);
+
 	g_free(info);
+	if( utf8!=NULL )
+	  g_free(utf8);
     }
 
-
-    /* Show ftp usage in the textview */
-    utf8 = g_locale_to_utf8(get_buffer, strlen(get_buffer), NULL, NULL, NULL);
-    free(get_buffer);
-
-    transfer_textbuffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widgets->transfer_textview));
-    gtk_text_buffer_set_text(transfer_textbuffer, utf8, strlen(utf8));
-    /* Get the start iter from the buffer */
-    gtk_text_buffer_get_start_iter(GTK_TEXT_BUFFER(transfer_textbuffer), &transfer_iter);
-    /* Create a mark at the start of the buffer specified by xfer_iter */
-    mark = gtk_text_buffer_create_mark(GTK_TEXT_BUFFER(transfer_textbuffer), mark_name, &transfer_iter, FALSE);
-    /* Move the mark on screen then scroll to it */
-    gtk_text_view_move_mark_onscreen(GTK_TEXT_VIEW(widgets->transfer_textview), mark);
-    gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(widgets->transfer_textview), mark, 0.4, TRUE, 0.0, 0.0);
-    gtk_text_buffer_delete_mark(GTK_TEXT_BUFFER(transfer_textbuffer), mark);
-
-    if( utf8!=NULL )
-      g_free(utf8);
-
     return(TRUE);
 }
