@@ -46,8 +46,6 @@ typedef struct w
   GtkWidget *notebook_vbox6;
   GtkWidget *notebook_vbox7;
   
-//  GtkWidget *gadmin_proftpd_version_label;
-//  GtkWidget *proftpd_version_label;
   GtkWidget *status_label;
   GtkWidget *version_label;
 
@@ -61,6 +59,8 @@ typedef struct w
   GtkWidget *server_set_spinbutton[NUM_SERVERTAB_SPINBUTTONS];
   GtkWidget *server_set_combo[NUM_SERVERTAB_COMBOS];
   GtkWidget *srv_set_table;
+  /* Enable virtual user management */
+  GtkWidget *server_set_check_virtual;
 
   /* The user tab's widgets */
   GtkWidget *user_treeview;
@@ -78,8 +78,6 @@ typedef struct w
   GtkWidget *useradd_username_entry;
   GtkWidget *useradd_password_entry;
   GtkWidget *useradd_homedir_entry;
-//  GtkWidget *useradd_upload_checkbutton;
-//  GtkWidget *useradd_uploaddir_entry;
   GtkWidget *useradd_usercomment_entry;
   GtkWidget *useradd_shell_combo;
   GtkWidget *useradd_uploaddir_checkbutton;  
@@ -96,11 +94,13 @@ typedef struct w
   GtkWidget *default_conf_question_window;
   
   /* The transfer tabs widgets */
-  GtkWidget *transfer_textview;
-  GtkWidget *total_bandwidth_entry;
-  GtkWidget *total_incoming_entry;
-  GtkWidget *total_outgoing_entry;
+  GtkWidget *transfer_treeview;
+  GtkListStore *transfer_store;
+  GtkWidget *total_bandwidth_label;
+  GtkWidget *total_incoming_label;
+  GtkWidget *total_outgoing_label;
   GtkWidget *kick_entry;
+  GtkWidget *transfer_pid_checkbutton;
 
   /* The disc tabs widgets */
   GtkWidget *disc_treeview;
